// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "VNEvent.hpp"

namespace Dawn {
  class VNChoiceEvent : public VNEvent {
    public:
      std::string text;
      std::string key;
      std::map<std::string, std::string> choices;
      int32_t choice = 0;

    protected:
      void onStart() override {
        choice = 0;

        std::cout << "CHOICE: " << text << std::endl;
        for(auto& choice : choices) {
          std::cout << "  " << choice.first << ": " << choice.second << std::endl;
        }

        useEvent([&](float_t delta) {
          auto im = &getScene()->game->inputManager;
          if(im->isPressed(INPUT_BIND_ACCEPT)) {
            auto it = choices.begin();
            std::advance(it, choice);
            std::string choiceMade = it->first;
            std::cout << "Choice made " << choiceMade << std::endl;
            this->manager->setFlag(this->key, choiceMade);
            this->next();
          } else if(im->isPressed(INPUT_BIND_NEGATIVE_Y)) {
            choice = mathClamp<int32_t>((choice - 1), 0, choices.size() - 1);
            std::cout << "Current choice: state" << choice << std::endl; 
          } else if(im->isPressed(INPUT_BIND_POSITIVE_Y)) {
            choice = mathClamp<int32_t>((choice + 1), 0, choices.size() - 1);
            std::cout << "Current choice: state" << choice << std::endl; 
          }
        }, getScene()->eventSceneUpdate);
      }
  };
}